#!/usr/bin/env python
# encoding: utf-8
"""Handle transfer participation to participant."""
import datetime
import logging
import os.path

from ringo.lib.helpers import get_app_location
from ringo.scripts.db import get_session
from efa.scripts.transfer_pn import transfer_pn_to_pt
from efa.model.journal import Journal
from ec.model.participant import MatchingParticipant, Participant
from ec.model.participation import ECParticipation

import transaction


log = logging.getLogger(__name__)


def add_transfer_parser(subparsers, parent):
    """add transfer_matched_participation parser"""
    transfer_parser = subparsers.add_parser(
        "transfer_matched_participation", parents=[parent],
        help="Transfer a matching participation to a participant")
    # three arguments
    transfer_parser.add_argument("--participation_id", type=int,
                                 help=("The ID of the participation "
                                       "which should be transfered."),
                                 required=True)

    help_msg_dppid = u"The personal id of the destination participant."
    transfer_parser.add_argument("--dest_participant_personal_id", type=str,
                                 help=help_msg_dppid, required=True)

    help_msg_sppid = u"The personal id of the source participant."
    transfer_parser.add_argument("--src_participant_personal_id", type=str,
                                 help=help_msg_sppid, required=True)

    transfer_parser.set_defaults(func=handle_transfer)


def handle_transfer(args):
    """Handle to transfer a pn to a pt, update journal, check matching_pts."""
    session = get_session_by_app_and_config(args.app, args.config)
    try:
        check_participation_at_src(session, args.src_participant_personal_id,
                                   args.participation_id)
        check_matching(session, args.src_participant_personal_id,
                       args.dest_participant_personal_id)
        transfer_pn_to_pt(session, args.participation_id,
                          args.dest_participant_personal_id)
        add_journal(session, args.participation_id,
                    args.src_participant_personal_id,
                    args.dest_participant_personal_id)
        remove_matchings(session, args.src_participant_personal_id)
        transaction.commit()
    except Exception as exp:
        transaction.abort()
        log.exception(exp)


def get_session_by_app_and_config(app, config):
    """uses app and config to open a session for the configured database."""
    path = []
    path.append(get_app_location(app))
    path.append(config)
    session = get_session(os.path.join(*path))
    return session


def check_participation_at_src(session, src_pid, pn_id):
    """Get the participant from the src_pid (source participant personal id)
    and check if this participant has only one participation and if
    this one particants participation has the ID equals pn_id."""
    participant = session.query(Participant).filter(
        Participant.personal_id == src_pid).one()
    if len(participant.participations) != 1:
        raise Exception("Participant has not one participation.")
    if participant.participations[0].id != pn_id:
        raise Exception("Participation of Participant doesn't match.")


def get_matching(session, src_pid, dest_pid):
    """Gets src_participant from the src_pid (personal id) and gets the
    dest_participant fro, the dest_pid (personal id), gets MatchingParticipant
    then with src_participant.id and dest_participant.id.
    Returns matching"""
    src_participant = session.query(Participant).filter(
        Participant.personal_id == src_pid).one()
    dest_participant = session.query(Participant).filter(
        Participant.personal_id == dest_pid).one()

    matching = session.query(MatchingParticipant).filter(
        MatchingParticipant.participant_id == src_participant.id).filter(
        MatchingParticipant.matching_participant_id == dest_participant.id) \
        .one()
    return matching


def check_matching(session, src_pid, dest_pid):
    """Gets matching and
    checks the status to be 2. This is the transfer status in the
    MatchingParticipant table."""

    matching = get_matching(session, src_pid, dest_pid)
    if matching.status != 2:
        raise Exception(u"Matching has status {} and not 2.".format(
            matching.status))


def add_journal(session, participation_id, src_pid, dest_pid):
    """Gets participation,
    matching (MatchingParticipant object).
    Uses the matching.comment to extract the user, who said, this should
    be transfered, and the datetime. This is saved in username and date_str.

    The main part is to generate the Journal entry."""

    # getting all needed information for the journal.
    participation = session.query(ECParticipation).get(participation_id)
    matching = get_matching(session, src_pid, dest_pid)
    username = matching.comment[30:].split(",")[0]
    date_str = ",".join(matching.comment[30:].split(",")[1:])

    # Now start to generate the Journal item, set participation, date is today,
    # description is generated from src_pid, username and date_str.
    journal = Journal()
    journal.participation = participation
    journal.date = datetime.date.today()
    journal.description = (u"Teilnahme von {} übertragen nach Prüfung durch"
                           u" {} vom {}").format(src_pid, username, date_str)

    # we want it to be written into the db.
    session.add(journal)


def remove_matchings(session, src_pid):
    """Remove the matchings from the source participant, so it can be
    deleted."""
    src_participant = session.query(Participant).filter(
        Participant.personal_id == src_pid).one()
    matchings = session.query(MatchingParticipant).filter(
        MatchingParticipant.participant_id == src_participant.id).all()
    for matching in matchings:
        session.delete(matching)
